<?php
/**
 * @file
 * Contains \Drupal\views_autocomplete_api\Controller\ViewAutocompleteController.
 *
 */

namespace Drupal\views_autocomplete_api\Controller;

//Use the necessary classes
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\views\Views;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class ViewAutocompleteController
 * @package Drupal\view_autocomplete_api\Controller
 */
class ViewAutocompleteController extends ControllerBase {

  /**
   * @param $view_name
   * @return JsonResponse
   */
  public function getViewsDataJson($view_name, Request $request) {
    $search = $request->query->get('q');
    // Load view
    /** @var \Drupal\views\ViewEntityInterface $view */
    if (strpos($view_name, ',')) {
      $view_list = explode(',', $view_name);
    }
    else {
      $view_list = array($view_name);
    }

    // Initialise the view data (the data which we returned in the fields autocomplete).
    $viewData = array();
    // Initialise the view data which we formatted to return a correct key-Value
    $viewDataFormatted = array();
    foreach ($view_list AS $view_name) {
      $view = \Drupal::entityManager()->getStorage('view')->load($view_name);
      if ($view) {
        $executable = $view->getExecutable();
        // Get the default (master) display definition.
        $executable->setDisplay('default');
        $view_display = $view->getDisplay('default');
        // Check permission to the view display default(master)
        if ($this->access($view_name, $view_display)) {
          // Loop on each exposed filter.
          $filters = array();
          //$executable->display_handler->options['cache'] = NULL;
          foreach ($executable->display_handler->options['filters'] AS $key => $options) {
            if ($options['exposed']) {
              $filters[$key] = $search;
            }
          }
          // Set request to auto-fill exposed filters.
          $query = $request->query;
          $query->add($filters);
          // Execute the view to get results.
          $executable->render();
          //Gets the current style plugin object.
          $currentStylePlugin = $executable->getStyle();
          // Transform the current style plugin object to array.
          $stylePlugin = (array) $currentStylePlugin;
          // The rendered fields array contains all fields of the view.
          $rendered_fields = array_values($this->filterArray($stylePlugin))[6];

          //Add Header if exist.
          if ($this->getData($rendered_fields, $search)) {
            $header = $this->getHeader($executable->display_handler->options['header'], $search);
            $data = $this->getData($rendered_fields, $search);
           // $footer = $this->getFooter($executable->display_handler->options['footer'], $search);
            $viewDataFormatted['key'] = array_merge($header,$data);
            $viewData[]= $viewDataFormatted;
         }
          else {
            $viewData[] = $this->getEmpty($executable->display_handler->options['empty']);
          }
        }
      }
    }


    return new JsonResponse($this->filterTable($viewData));
  }

  /** Reorder the autocomplete results.
   * @param $stylePlugin
   * @return array
   */
  function filterArray($stylePlugin) {
    $new_style_plugin = array();
    foreach ($stylePlugin as $key => $row) {
      $new_key = str_replace('*', '', $key);
      $new_style_plugin[$new_key] = $row;
    }
    return $new_style_plugin;
  }

  /**
   * Combine all results in one array.
   * @param $viewDataFormatted
   * @return array
   */
  function filterTable($viewDataFormatted) {
    $res = array();
    foreach ($viewDataFormatted as $v) {
      foreach ($v as $index => $data_views) {
        if (is_array($data_views)) {
          foreach ($data_views as $item) {
            $res[] = $item;
          }
        }
        else {
          $res[] = $data_views;
        }
      }
    }
    return $res;
  }

  /** Get header of the view if exist.
   * @param $footer
   * @param $search
   * @return array
   */
  protected function getHeader($header, $search) {
    $viewDataFormatted = array();
    if (is_array($header)) {
      if (strpos($header['area']['content']['value'], '[autocomplete]')) {
        $header_replace = str_replace('[autocomplete]', $search, $header['area']['content']['value']);
      }
      else {
        $header_replace = $header['area']['content']['value'];
      }
      $viewDataFormatted[] = '<span class="group-header">' . $header_replace . '</span>';
    }
    return $viewDataFormatted;
  }

  /** Get footer of the view if exist.
   * @param $footer
   * @param $search
   * @return array
   */
  protected function  getFooter($footer, $search) {
    $viewDataFormatted = array();
    if (is_array($footer)) {
      if (strpos($footer['area']['content']['value'], '[autocomplete]')) {
        $footer_replace = str_replace('[autocomplete]', $search, $footer['area']['content']['value']);
      }
      else {
        $footer_replace = $footer['area']['content']['value'];
      }
      $viewDataFormatted[] = '<span class="group-footer">' . $footer_replace . '</span>';
      return $viewDataFormatted;
    }
  }
  /** Get No results behaviour of the view if exist.
   * @param $empty
   * @return array
   */
  protected function getEmpty($empty) {
    if (!empty($empty)) {
      $viewDataFormatted[] = '<span class="group-empty">' . $empty['area']['content']['value'] . '</span>';
      return $viewDataFormatted;
    }
  }

  /** Return data.
   * @param $rendered_fields
   * @return array
   */
  protected function getData($rendered_fields, $search) {
    $viewDataFormatted = array();
      // The String Which search for.
    if ($rendered_fields != NULL) {
      //dpm($rendered_fields);
      foreach ($rendered_fields as $row) {
        // Content of rendered fields.
        $rowValues = array_values($row);
       // dpm($rowValues);
        $count = count($rowValues);

        $key = $val = $rowValues[count($row) - 2];
        //Take the last field to allow to call more that one and "Rewrite field" and call them all.
        if ($count > 1) {
          $val = $rowValues[$count - 1];
        }
        $viewData['value'] = $key;
          //Highlight search word
          $rendered = $this->highlightStr($val, $search);
        $viewData['label'] = $rendered;
        $viewDataFormatted[] = $viewData;
      }
      return ($viewDataFormatted);
    }
  }

    /**
     * @param $haystack
     * @param $needle
     * @return $haystack
     */
    public function highlightStr($haystack, $needle) {
        // return $haystack if there is no highlight color or strings given, nothing to do.
        if (strlen($haystack) < 1 || strlen($needle) < 1) {
            return $haystack;
        }

//        preg_match_all("/$needle+/i", $haystack, $matches);

        $patterns[0] = "/(?![^<]*>)$needle+/i";
        $replacements[0] = '<span>' .$needle. '</span>';

        # addon check translitered search query
        $translitered_match =  $this->removeAccents($needle);
        if ($needle != $translitered_match) {
          $patterns[1] = "/(?![^<]*>)$translitered_match+/i";
          $replacements[1] = '<span>' .$translitered_match. '</span>';
        }

        $haystack = preg_replace($patterns, $replacements, $haystack);

      /*
        print_r($matches);
        if (is_array($matches[0]) && count($matches[0]) >= 1) {
            foreach ($matches[0] as $match) {
              $haystack = str_ireplace($match, '<span>'.$match.'</span>', $haystack);

              # addon check translitered search query
              $translitered_match =  $this->removeAccents($match);
              if ($match != $translitered_match) {
//                echo $match.' / '.$translitered_match;
                $haystack = str_ireplace($translitered_match, '<span>' . $translitered_match . '</span>', $haystack);
              }
            }
        }
        */
        return $haystack;
    }


  /**
   * @param $view_name
   * @param $view_display
   * @return AccessResult
   */
  public function access($view_name, $view_display) {
    // Determine if the given user has access to the view. Note that
    // this sets the display handler if it hasn't been.
    $view = Views::getView($view_name);
    $current_user = \Drupal::currentUser();
    if ($view->access($view_display) && $current_user->hasPermission('administer views')) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  public function removeAccents($str, $encoding='utf-8') {
      // transformer les caractères accentués en entités HTML
      $str = htmlentities($str, ENT_NOQUOTES, $encoding);

      // remplacer les entités HTML pour avoir juste le premier caractères non accentués
      // Exemple : "&ecute;" => "e", "&Ecute;" => "E", "Ã " => "a" ...
      $str = preg_replace('#&([A-za-z])(?:acute|grave|cedil|circ|orn|ring|slash|th|tilde|uml);#', '\1', $str);

      // Remplacer les ligatures tel que : Œ, Æ ...
      // Exemple "Å“" => "oe"
      $str = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $str);
      // Supprimer tout le reste
      $str = preg_replace('#&[^;]+;#', '', $str);

    return $str;
  }

}


